import * as i0 from '@angular/core';
import { InjectionToken, Injectable, NgModule } from '@angular/core';
import { setupListeners, QueryStatus, skipToken, buildCreateApi, coreModule } from '@reduxjs/toolkit/query';
import { take, map, distinctUntilChanged, switchMap, tap, shareReplay, finalize } from 'rxjs/operators';
import * as i1 from '@ngrx/store';
import { createSelectorFactory, resultMemoize } from '@ngrx/store';
import { isObservable, of, combineLatest, BehaviorSubject } from 'rxjs';

const defaultConfig = {
    setupListeners: false,
};
const STORE_RTK_QUERY_CONFIG = new InjectionToken('STORE_RTK_QUERY_CONFIG');

let service;
let delayedActions = [];
function dispatch(action) {
    var _a;
    if (typeof action === 'function') {
        return action(dispatch, getState, {});
    }
    // Middleware dispatch actions before Store starts
    if (service && ((_a = Object.keys(getState())) === null || _a === void 0 ? void 0 : _a.length)) {
        delayedActions.map((delayedAction) => service.dispatch(delayedAction));
        delayedActions = [];
        service.dispatch(action);
    }
    else {
        delayedActions.push(action);
    }
    return action;
}
function getState() {
    return service === null || service === void 0 ? void 0 : service.getState();
}
function select(mapFn) {
    return service === null || service === void 0 ? void 0 : service.select(mapFn);
}
class ThunkService {
    constructor(store) {
        this.store = store;
        service = this;
    }
    getState() {
        let state = {};
        // eslint-disable-next-line ngrx/no-store-subscription
        this.store.pipe(take(1)).subscribe((res) => (state = res));
        return state;
    }
    dispatch(action) {
        var _a;
        (_a = this.store) === null || _a === void 0 ? void 0 : _a.dispatch(action);
    }
    select(mapFn) {
        return this.store.select(mapFn);
    }
}
/** @nocollapse */ /** @nocollapse */ ThunkService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: ThunkService, deps: [{ token: i1.Store }], target: i0.ɵɵFactoryTarget.Injectable });
/** @nocollapse */ /** @nocollapse */ ThunkService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: ThunkService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: ThunkService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: i1.Store }]; } });

class StoreRtkQueryModule {
    constructor(thunkService) {
        this.thunkService = thunkService;
    }
    static forRoot(config) {
        const moduleConfig = Object.assign(Object.assign({}, defaultConfig), config);
        if (moduleConfig.setupListeners) {
            setupListeners(dispatch, moduleConfig.customHandler);
        }
        return {
            ngModule: StoreRtkQueryModule,
            providers: [{ provide: STORE_RTK_QUERY_CONFIG, useValue: moduleConfig }, ThunkService],
        };
    }
}
/** @nocollapse */ /** @nocollapse */ StoreRtkQueryModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: StoreRtkQueryModule, deps: [{ token: ThunkService }], target: i0.ɵɵFactoryTarget.NgModule });
/** @nocollapse */ /** @nocollapse */ StoreRtkQueryModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: StoreRtkQueryModule });
/** @nocollapse */ /** @nocollapse */ StoreRtkQueryModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: StoreRtkQueryModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.1.1", ngImport: i0, type: StoreRtkQueryModule, decorators: [{
            type: NgModule,
            args: [{}]
        }], ctorParameters: function () { return [{ type: ThunkService }]; } });

const UNINITIALIZED_VALUE = Symbol();

function capitalize(str) {
    return str.replace(str[0], str[0].toUpperCase());
}

function is(x, y) {
    if (x === y) {
        return x !== 0 || y !== 0 || 1 / x === 1 / y;
    }
    else {
        return x !== x && y !== y;
    }
}
function shallowEqual(objA, objB) {
    if (is(objA, objB)) {
        return true;
    }
    if (typeof objA !== 'object' || objA === null || typeof objB !== 'object' || objB === null) {
        return false;
    }
    const keysA = Object.keys(objA);
    const keysB = Object.keys(objB);
    if (keysA.length !== keysB.length) {
        return false;
    }
    // eslint-disable-next-line @typescript-eslint/prefer-for-of
    for (let i = 0; i < keysA.length; i++) {
        if (!Object.prototype.hasOwnProperty.call(objB, keysA[i]) || !is(objA[keysA[i]], objB[keysA[i]])) {
            return false;
        }
    }
    return true;
}

function safeAssign(target, ...args) {
    Object.assign(target, ...args);
}

function useStableQueryArgs(queryArgs, serialize, endpointDefinition, endpointName, cacheRef = {}) {
    const incoming = {
        queryArgs,
        serialized: typeof queryArgs == 'object' ? serialize({ queryArgs, endpointDefinition, endpointName }) : queryArgs,
    };
    if (!cacheRef.current || cacheRef.current.serialized !== incoming.serialized) {
        cacheRef.current = incoming;
    }
    return cacheRef.current.queryArgs;
}

const defaultQueryStateSelector = (x) => x;
const defaultMutationStateSelector = (x) => x;
/**
 * Wrapper around `defaultQueryStateSelector` to be used in `useQuery`.
 * We want the initial render to already come back with
 * `{ isUninitialized: false, isFetching: true, isLoading: true }`
 * to prevent that the library user has to do an additional check for `isUninitialized`/
 */
const noPendingQueryStateSelector = (selected) => {
    if (selected.isUninitialized) {
        return Object.assign(Object.assign({}, selected), { isUninitialized: false, isFetching: true, isLoading: selected.data !== undefined ? false : true, status: QueryStatus.pending });
    }
    return selected;
};
/**
 *
 * @param opts.api - An API with defined endpoints to create hooks for
 * @param opts.moduleOptions.useDispatch - The version of the `useDispatch` hook to be used
 * @param opts.moduleOptions.useSelector - The version of the `useSelector` hook to be used
 * @returns An object containing functions to generate hooks based on an endpoint
 */
function buildHooks({ api, moduleOptions: { useDispatch: dispatch, useSelector }, serializeQueryArgs, context, }) {
    return { buildQueryHooks, buildMutationHook, usePrefetch };
    function queryStatePreSelector(currentState, lastResult, queryArgs) {
        // if we had a last result and the current result is uninitialized,
        // we might have called `api.util.resetApiState`
        // in this case, reset the hook
        if ((lastResult === null || lastResult === void 0 ? void 0 : lastResult.endpointName) && currentState.isUninitialized) {
            const { endpointName } = lastResult;
            const endpointDefinition = context.endpointDefinitions[endpointName];
            if (serializeQueryArgs({
                queryArgs: lastResult.originalArgs,
                endpointDefinition,
                endpointName,
            }) ===
                serializeQueryArgs({
                    queryArgs,
                    endpointDefinition,
                    endpointName,
                }))
                lastResult = undefined;
        }
        // data is the last known good request result we have tracked
        // or if none has been tracked yet the last good result for the current args
        let data = currentState.isSuccess ? currentState.data : lastResult === null || lastResult === void 0 ? void 0 : lastResult.data;
        if (data === undefined)
            data = currentState.data;
        const hasData = data !== undefined;
        // isFetching = true any time a request is in flight
        const isFetching = currentState.isLoading;
        // isLoading = true only when loading while no data is present yet (initial load with no data in the cache)
        const isLoading = !hasData && isFetching;
        // isSuccess = true when data is present
        const isSuccess = currentState.isSuccess || (isFetching && hasData);
        return Object.assign(Object.assign({}, currentState), { data, currentData: currentState.data, isFetching,
            isLoading,
            isSuccess });
    }
    function usePrefetch(endpointName, defaultOptions) {
        return (arg, options) => dispatch(api.util.prefetch(endpointName, arg, Object.assign(Object.assign({}, defaultOptions), options)));
    }
    function buildQueryHooks(name) {
        const { initiate, select } = api.endpoints[name];
        const useQuerySubscription = (arg, { refetchOnReconnect, refetchOnFocus, refetchOnMountOrArgChange, skip = false, pollingInterval = 0 } = {}, promiseRef = {}, argCacheRef = {}) => {
            var _a;
            const stableArg = useStableQueryArgs(skip ? skipToken : arg, serializeQueryArgs, context.endpointDefinitions[name], name, argCacheRef);
            const subscriptionOptions = { refetchOnReconnect, refetchOnFocus, pollingInterval };
            const { queryCacheKey, requestId } = promiseRef.current || {};
            useSelector((state) => { var _a; return !!queryCacheKey && !!requestId && !((_a = state[api.reducerPath].subscriptions[queryCacheKey]) === null || _a === void 0 ? void 0 : _a[requestId]); })
                .pipe(take(1))
                .subscribe((subscriptionRemoved) => {
                var _a;
                if (subscriptionRemoved) {
                    (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                    promiseRef.current = undefined;
                }
            });
            if (stableArg !== skipToken && stableArg !== UNINITIALIZED_VALUE) {
                const lastPromise = promiseRef === null || promiseRef === void 0 ? void 0 : promiseRef.current;
                const lastSubscriptionOptions = (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.subscriptionOptions;
                if (!lastPromise || !shallowEqual(lastPromise.arg, stableArg)) {
                    lastPromise === null || lastPromise === void 0 ? void 0 : lastPromise.unsubscribe();
                    promiseRef.current = dispatch(initiate(stableArg, { subscriptionOptions, forceRefetch: refetchOnMountOrArgChange }));
                }
                else if (!shallowEqual(subscriptionOptions, lastSubscriptionOptions)) {
                    lastPromise.updateSubscriptionOptions(subscriptionOptions);
                }
            }
            return {
                /**
                 * A method to manually refetch data for the query
                 */
                refetch: () => { var _a; return void ((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.refetch()); },
            };
        };
        const useLazyQuerySubscription = ({ refetchOnReconnect, refetchOnFocus, pollingInterval = 0 } = {}, promiseRef = {}) => {
            var _a, _b;
            let argState = UNINITIALIZED_VALUE;
            const subscriptionOptions = { refetchOnReconnect, refetchOnFocus, pollingInterval };
            const lastSubscriptionOptions = (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.subscriptionOptions;
            if (!shallowEqual(subscriptionOptions, lastSubscriptionOptions)) {
                (_b = promiseRef.current) === null || _b === void 0 ? void 0 : _b.updateSubscriptionOptions(subscriptionOptions);
            }
            const trigger = (arg, { preferCacheValue = false } = {}) => {
                var _a;
                (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe();
                const promise = dispatch(initiate(arg, { subscriptionOptions, forceRefetch: !preferCacheValue }));
                promiseRef.current = promise;
                argState = arg;
                return promise;
            };
            /* if "cleanup on unmount" was triggered from a fast refresh, we want to reinstate the query */
            if (argState !== UNINITIALIZED_VALUE && !promiseRef.current) {
                trigger(argState, { preferCacheValue: true });
            }
            return [trigger, argState];
        };
        const useQueryState = (arg, { skip = false, selectFromResult = defaultQueryStateSelector } = {}, lastValue = {}, argCacheRef = {}) => {
            const arg$ = isObservable(arg) ? arg : of(arg);
            return arg$.pipe(map((currentArg) => useStableQueryArgs(skip || currentArg === UNINITIALIZED_VALUE ? skipToken : currentArg, serializeQueryArgs, context.endpointDefinitions[name], name, argCacheRef)), distinctUntilChanged(shallowEqual), switchMap((stableArg) => {
                const selectDefaultResult = createSelectorFactory((projector) => resultMemoize(projector, shallowEqual))(select(stableArg), (subState) => queryStatePreSelector(subState, lastValue.current, stableArg));
                const querySelector = createSelectorFactory((projector) => resultMemoize(projector, shallowEqual))(selectDefaultResult, selectFromResult);
                return useSelector((state) => querySelector(state)).pipe(tap(() => (lastValue.current = selectDefaultResult(getState()))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }));
        };
        const useQuery = (arg, options) => {
            // Refs
            const promiseRef = {};
            const lastValue = {};
            const argQueryRef = {};
            const argRef = {};
            const arg$ = isObservable(arg) ? arg : of(arg);
            const options$ = isObservable(options) ? options : of(options);
            return combineLatest([
                arg$.pipe(map((currentArg) => useStableQueryArgs(currentArg === UNINITIALIZED_VALUE ? skipToken : currentArg, serializeQueryArgs, context.endpointDefinitions[name], name, argQueryRef)), distinctUntilChanged(shallowEqual)),
                options$.pipe(distinctUntilChanged((prev, curr) => shallowEqual(prev, curr))),
            ]).pipe(switchMap(([stableArg, currentOptions]) => {
                const querySubscriptionResults = useQuerySubscription(stableArg, currentOptions, promiseRef, argRef);
                const queryStateResults$ = useQueryState(stableArg, Object.assign({ selectFromResult: stableArg === skipToken || (currentOptions === null || currentOptions === void 0 ? void 0 : currentOptions.skip) ? undefined : noPendingQueryStateSelector }, currentOptions), lastValue, argRef);
                return queryStateResults$.pipe(map((queryState) => (Object.assign(Object.assign({}, queryState), querySubscriptionResults))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }), finalize(() => {
                var _a;
                void ((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe());
                promiseRef.current = undefined;
            }));
        };
        const useLazyQuery = (options) => {
            // Refs
            const promiseRef = {};
            const lastValue = {};
            const triggerRef = {};
            const argRef = {};
            const infoSubject = new BehaviorSubject({ lastArg: UNINITIALIZED_VALUE });
            const info$ = infoSubject.asObservable();
            const options$ = isObservable(options) ? options : of(options);
            const state$ = combineLatest([
                options$.pipe(distinctUntilChanged((prev, curr) => shallowEqual(prev, curr)), tap((currentOptions) => {
                    const [trigger] = useLazyQuerySubscription(currentOptions, promiseRef);
                    triggerRef.current = trigger;
                })),
                info$.pipe(tap(({ lastArg, extra }) => {
                    var _a;
                    if (lastArg !== UNINITIALIZED_VALUE) {
                        (_a = triggerRef.current) === null || _a === void 0 ? void 0 : _a.call(triggerRef, lastArg, extra);
                    }
                }), map(({ lastArg }) => lastArg), distinctUntilChanged(shallowEqual)),
            ]).pipe(switchMap(([currentOptions, currentArg]) => useQueryState(currentArg, Object.assign(Object.assign({}, currentOptions), { skip: currentArg === UNINITIALIZED_VALUE }), lastValue, argRef)), shareReplay({
                bufferSize: 1,
                refCount: true,
            }), finalize(() => {
                var _a;
                void ((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.unsubscribe());
                promiseRef.current = undefined;
            }));
            return {
                fetch: (arg, extra) => {
                    infoSubject.next({ lastArg: arg, extra });
                    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
                    return promiseRef.current;
                },
                state$,
                lastArg$: info$.pipe(map(({ lastArg }) => lastArg)),
            };
        };
        return {
            useQueryState,
            useQuerySubscription,
            useLazyQuerySubscription,
            useLazyQuery,
            useQuery,
        };
    }
    function buildMutationHook(name) {
        const { initiate, select } = api.endpoints[name];
        return ({ selectFromResult = defaultMutationStateSelector, fixedCacheKey } = {}) => {
            const promiseRef = {};
            const requestIdSubject = new BehaviorSubject(undefined);
            const requestId$ = requestIdSubject.asObservable();
            const triggerMutation = (arg) => {
                var _a;
                const promise = dispatch(initiate(arg, { fixedCacheKey }));
                if (!((_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.arg.fixedCacheKey)) {
                    removePrevMutation();
                }
                promiseRef.current = promise;
                requestIdSubject.next(promise.requestId);
                return promise;
            };
            const reset = () => {
                removePrevMutation();
                requestIdSubject.next(undefined);
            };
            const removePrevMutation = () => {
                if (promiseRef.current) {
                    dispatch(api.internalActions.removeMutationResult({ requestId: promiseRef.current.requestId, fixedCacheKey }));
                    promiseRef.current = undefined;
                }
            };
            const state$ = requestId$.pipe(finalize(() => {
                var _a;
                (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.reset();
                promiseRef.current = undefined;
            }), distinctUntilChanged(shallowEqual), switchMap((requestId) => {
                var _a;
                const mutationSelector = createSelectorFactory((projector) => resultMemoize(projector, shallowEqual))(select(requestId ? { fixedCacheKey, requestId } : skipToken), (subState) => selectFromResult(subState));
                const currentState = useSelector((state) => mutationSelector(state));
                const originalArgs = fixedCacheKey == null ? (_a = promiseRef.current) === null || _a === void 0 ? void 0 : _a.arg.originalArgs : undefined;
                return currentState.pipe(map((mutationState) => (Object.assign(Object.assign({}, mutationState), { originalArgs, reset }))));
            }), shareReplay({
                bufferSize: 1,
                refCount: true,
            }));
            return { dispatch: triggerMutation, state$ };
        };
    }
}

function buildMetaReducer({ api, moduleOptions: { useDispatch: dispatch }, }) {
    const anyApi = api;
    let nextState;
    const getState = () => nextState[anyApi.reducerPath]
        ? nextState
        : // Query inside forFeature (Code splitting)
            { [anyApi.reducerPath]: nextState };
    const middleware = anyApi.middleware({ dispatch, getState })(getState);
    return function (reducer) {
        return function (state, action) {
            nextState = reducer(state, action);
            middleware(action);
            return nextState;
        };
    };
}

function isQueryDefinition(e) {
    return e.type === 'query';
}
function isMutationDefinition(e) {
    return e.type === 'mutation';
}

const angularHooksModuleName = /* @__PURE__ */ Symbol();
/**
 * Creates a module that generates hooks from endpoints, for use with `buildCreateApi`.
 *
 * @returns A module for use with `buildCreateApi`
 */
const angularHooksModule = ({ useDispatch = dispatch, useSelector = select, getState: getState$1 = getState, } = {}) => ({
    name: angularHooksModuleName,
    init(api, { serializeQueryArgs }, context) {
        const anyApi = api;
        const { buildQueryHooks, buildMutationHook, usePrefetch } = buildHooks({
            api,
            moduleOptions: { useDispatch, useSelector, getState: getState$1 },
            serializeQueryArgs,
            context,
        });
        const metareducer = buildMetaReducer({ api, moduleOptions: { useDispatch } });
        safeAssign(anyApi, { usePrefetch });
        safeAssign(anyApi, { metareducer });
        return {
            injectEndpoint(endpointName, definition) {
                if (isQueryDefinition(definition)) {
                    const { useQuery, useLazyQuery, useLazyQuerySubscription, useQueryState, useQuerySubscription } = buildQueryHooks(endpointName);
                    safeAssign(anyApi.endpoints[endpointName], {
                        useQuery,
                        useLazyQuery,
                        useLazyQuerySubscription,
                        useQueryState,
                        useQuerySubscription,
                    });
                    api[`use${capitalize(endpointName)}Query`] = useQuery;
                    api[`useLazy${capitalize(endpointName)}Query`] = useLazyQuery;
                }
                else if (isMutationDefinition(definition)) {
                    const useMutation = buildMutationHook(endpointName);
                    safeAssign(anyApi.endpoints[endpointName], { useMutation });
                    api[`use${capitalize(endpointName)}Mutation`] = useMutation;
                }
            },
        };
    },
});
const createApi = /* @__PURE__ */ buildCreateApi(coreModule(), angularHooksModule());

/*
 * Public API Surface of ngrx-rtk-query
 */

/**
 * Generated bundle index. Do not edit.
 */

export { StoreRtkQueryModule, ThunkService, angularHooksModule, createApi, dispatch };
//# sourceMappingURL=ngrx-rtk-query.mjs.map
